
AWS: str = "AWS"
GCP: str = "GCP"
AZURE: str = "AZURE"
IBM: str = "IBM"
JIRA_ISSUE_NEW_STATE: str = 'New'
JIRA_ISSUE_REFINEMENT_STATE: str = 'Refinement'
JIRA_ISSUE_CLOSED_STATE: str = 'Closed'
JIRA_ISSUE_IN_PROGRESS_STATE: str = 'In Progress'
FIRST_CRO_ALERT: int = 5
SECOND_CRO_ALERT: int = 3
CLOSE_JIRA_TICKET: int = 0
DEFAULT_ROUND_DIGITS: int = 3
DATE_FORMAT: str = '%Y-%m-%d'
DURATION = 'Duration'
TICKET_ID = 'TicketId'
CLOUD_GOVERNANCE_ES_MAIL_INDEX = 'cloud-governance-mail-messages'
CRO_OVER_USAGE_ALERT = 'cro-over-usage-alert'
TIMESTAMP_DATE_FORMAT = "%Y-%m-%dT%H:%M:%S.%f"
OVER_USAGE_THRESHOLD = 0.05
SEND_ALERT_DAY = 3
